@@ -8,34 +8,25 @@ use bytes::Bytes;
 use futures::{Stream, TryStreamExt};
 
 use header::HeaderName;
+
 use interface::{BlobMeta, MetadataList, Query, QueryResponse, RoutingConfig};
 
 use hyper::{header, StatusCode};
 
 use mpart_async::client::MultipartRequest;
 
-use protocol::{
-    directory::{
-        auth::{LoginRequest, LoginResponse, RegisterRequest},
-        blobmeta::{GetMetaResponse, ListMetadataRequest},
-        routing::{GetRoutingConfigResponse, SetRoutingConfigRequest},
-        storage::ListStorageNodesResponse,
-    },
-    storage::PutResponse,
-};
+use protocol::directory::{auth::*, blobmeta::*, routing::*, storage::*};
+use protocol::storage::PutResponse;
+
 use reqwest::{Client as ReqwestClient, Request};
 
 use reqwest::Body;
 
 use serde::de::DeserializeOwned;
-use snafu::{ensure, ResultExt, Snafu};
 
-use crate::{
-    metadata_detector::{MetadataDetector, MetadataDetectorError},
-    parameters::HostConfig,
-    profile::ProfileError,
-    ClientBuilder, Config, Meta, Parameters,
-};
+use snafu::prelude::*;
+
+use crate::{ClientBuilder, Meta, Parameters};
 
 #[derive(Debug, Snafu)]
 pub enum ClientError {
@@ -69,12 +60,6 @@ pub enum ClientError {
     #[snafu(display("server returned an error: {}", message))]
     ServerReturnedError { message: String },
 
-    #[snafu(display("failed to load configuration: {}", source))]
-    ConfigLoadError { source: ProfileError },
-
-    #[snafu(display("missing profile '{}'", name))]
-    MissingProfile { name: String },
-
     #[snafu(display("did not get a redirect when expected"))]
     MissingRedirect,
 
@@ -83,19 +68,16 @@ pub enum ClientError {
 
     #[snafu(display("unknown error"))]
     UnknownError,
-
-    #[snafu(display("failed to instantiate the metadata detector: {}", source))]
-    MetadataDetectorErrorInstantiationError { source: MetadataDetectorError },
 }
 
 fn encode_metadata(meta: Meta) -> Result<String> {
-    let serialized_meta = serde_json::to_vec(&meta).context(MetaSerializationError { meta })?;
+    let serialized_meta = serde_json::to_vec(&meta).context(MetaSerializationSnafu { meta })?;
     Ok(base64::encode(&serialized_meta))
 }
 
 async fn extract_body<T: DeserializeOwned>(response: reqwest::Response) -> Result<T> {
-    let body_bytes = response.bytes().await.context(FetchBodyError)?;
-    serde_json::from_slice(body_bytes.as_ref()).context(ResponseDeserializationError)
+    let body_bytes = response.bytes().await.context(FetchBodySnafu)?;
+    serde_json::from_slice(body_bytes.as_ref()).context(ResponseDeserializationSnafu)
 }
 
 async fn extract_error(response: reqwest::Response) -> ClientError {
@@ -122,7 +104,6 @@ pub struct Client {
     max_retry_count: usize,
     retry_interval: Duration,
     token: String,
-    metadata_detector: Option<MetadataDetector>,
 }
 
 type Result<T> = std::result::Result<T, ClientError>;
@@ -132,39 +113,16 @@ impl Client {
     pub async fn new<S: Into<String>, U: Into<String>, P: Into<String>>(
         directory_host: S,
         username: U,
-        admin_password: P,
+        password: P,
     ) -> Result<Self> {
         Client::new_with_params(Parameters {
-            host_config: HostConfig::Host {
-                host: directory_host.into(),
-                username: username.into(),
-                admin_password: admin_password.into(),
-            },
+            host: directory_host.into(),
+            username: username.into(),
+            password: password.into(),
             pool_idle_timeout: Duration::from_secs(5),
             request_timeout: Duration::from_secs(60),
             max_retry_count: 20,
             retry_interval: Duration::from_millis(100),
-            metadata_detection: false,
-        })
-        .await
-    }
-
-    /// Create a new client from a configured profile with default settings.
-    ///
-    /// The profiles are read from
-    /// * `$XDG_CONFIG_HOME/menmos/client.toml` on Linux
-    /// * `~/Library/Application Support/menmos/client.toml` on MacOS
-    /// * `%APPDATA%\menmos\client.toml` on Windows
-    pub async fn new_with_profile<S: Into<String>>(profile: S) -> Result<Self> {
-        Self::new_with_params(Parameters {
-            host_config: HostConfig::Profile {
-                profile: profile.into(),
-            },
-            pool_idle_timeout: Duration::from_secs(5),
-            request_timeout: Duration::from_secs(60),
-            max_retry_count: 20,
-            retry_interval: Duration::from_millis(100),
-            metadata_detection: false,
         })
         .await
     }
@@ -180,43 +138,17 @@ impl Client {
             .timeout(params.request_timeout)
             .redirect(reqwest::redirect::Policy::none())
             .build()
-            .context(ClientBuildError)?;
+            .context(ClientBuildSnafu)?;
 
-        let (host, username, admin_password) = match params.host_config {
-            HostConfig::Host {
-                host,
-                username,
-                admin_password,
-            } => (host, username, admin_password),
-            HostConfig::Profile { profile } => {
-                let config = Config::load().context(ConfigLoadError)?;
-                let profile = config
-                    .profiles
-                    .get(&profile)
-                    .ok_or(ClientError::MissingProfile { name: profile })?;
-                (
-                    profile.host.clone(),
-                    profile.username.clone(),
-                    profile.password.clone(),
-                )
-            }
-        };
-
-        let token = Client::login(&client, &host, &username, &admin_password).await?;
-
-        let metadata_detector = if params.metadata_detection {
-            Some(MetadataDetector::new().context(MetadataDetectorErrorInstantiationError)?)
-        } else {
-            None
-        };
+        let token =
+            Client::login(&client, &params.host, &params.username, &params.password).await?;
 
         Ok(Self {
-            host,
+            host: params.host,
             client,
             max_retry_count: params.max_retry_count,
             retry_interval: params.retry_interval,
             token,
-            metadata_detector,
         })
     }
 
@@ -227,7 +159,7 @@ impl Client {
                 .client
                 .execute(req_fn()?)
                 .await
-                .context(RequestExecutionError)
+                .context(RequestExecutionSnafu)
             {
                 Ok(r) => return Ok(r),
                 Err(e) => {
@@ -267,14 +199,14 @@ impl Client {
                 .header(header::HeaderName::from_static("x-blob-meta"), encoded_meta)
                 .body(Body::wrap_stream(mpart))
                 .build()
-                .context(RequestBuildError)
+                .context(RequestBuildSnafu)
         } else {
             self.client
                 .post(url)
                 .bearer_auth(&self.token)
                 .header(header::HeaderName::from_static("x-blob-meta"), encoded_meta)
                 .build()
-                .context(RequestBuildError)
+                .context(RequestBuildSnafu)
         }
     }
 
@@ -283,11 +215,11 @@ impl Client {
             .client
             .execute(request)
             .await
-            .context(RequestExecutionError)?;
+            .context(RequestExecutionSnafu)?;
 
         ensure!(
             response.status() == StatusCode::TEMPORARY_REDIRECT,
-            MissingRedirect
+            MissingRedirectSnafu
         );
 
         let new_location = response
@@ -316,7 +248,7 @@ impl Client {
             })
             .send()
             .await
-            .context(RequestExecutionError)?;
+            .context(RequestExecutionSnafu)?;
 
         let resp: LoginResponse = extract(response).await?;
 
@@ -336,7 +268,7 @@ impl Client {
             })
             .send()
             .await
-            .context(RequestExecutionError)?;
+            .context(RequestExecutionSnafu)?;
 
         let resp: LoginResponse = extract(response).await?;
         Ok(resp.token)
@@ -355,7 +287,7 @@ impl Client {
             .bearer_auth(&self.token)
             .header(HeaderName::from_static("x-blob-meta"), meta_b64.clone())
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(redirect_req).await?;
 
@@ -366,7 +298,7 @@ impl Client {
                     .bearer_auth(&self.token)
                     .header(HeaderName::from_static("x-blob-meta"), &meta_b64)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -377,22 +309,16 @@ impl Client {
     async fn push_internal<P: AsRef<Path>>(
         &self,
         path: P,
-        mut meta: Meta,
+        meta: Meta,
         base_url: String,
     ) -> Result<String> {
         ensure!(
             path.as_ref().exists(),
-            FileDoesNotExist {
+            FileDoesNotExistSnafu {
                 path: PathBuf::from(path.as_ref())
             }
         );
 
-        if let Some(metadata_detector) = self.metadata_detector.as_ref() {
-            metadata_detector
-                .populate(&path, &mut meta)
-                .context(MetadataDetectorErrorInstantiationError)?;
-        }
-
         let mut url = base_url;
         let meta_b64 = encode_metadata(meta)?;
 
@@ -405,7 +331,7 @@ impl Client {
                 meta_b64.clone(),
             )
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         url = self.request_with_redirect(initial_redirect_request).await?;
 
@@ -424,7 +350,7 @@ impl Client {
         let url = format!("{}/health", self.host);
 
         let response = self
-            .execute(|| self.client.get(&url).build().context(RequestBuildError))
+            .execute(|| self.client.get(&url).build().context(RequestBuildSnafu))
             .await?;
 
         let status = response.status();
@@ -447,7 +373,7 @@ impl Client {
                     .get(&url)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -493,7 +419,7 @@ impl Client {
             .json(&ListMetadataRequest { tags, meta_keys })
             .send()
             .await
-            .context(RequestExecutionError)?;
+            .context(RequestExecutionSnafu)?;
 
         extract(response).await
     }
@@ -508,7 +434,7 @@ impl Client {
             .bearer_auth(&self.token)
             .json(&meta)
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(request).await?;
 
@@ -519,7 +445,7 @@ impl Client {
                     .bearer_auth(&self.token)
                     .json(&meta)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -541,7 +467,7 @@ impl Client {
             .post(&url)
             .bearer_auth(&self.token)
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(request).await?;
 
@@ -551,7 +477,7 @@ impl Client {
                     .post(&redirect_location)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -575,7 +501,7 @@ impl Client {
                 &format!("bytes={}-{}", offset, offset + (buffer.len() - 1) as u64),
             )
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(request).await?;
 
@@ -590,7 +516,7 @@ impl Client {
                     )
                     .body(buffer.clone())
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -615,7 +541,7 @@ impl Client {
                     .get(&url)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -632,7 +558,7 @@ impl Client {
             .get(&url)
             .bearer_auth(&self.token)
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(redirect_request).await?;
 
@@ -642,7 +568,7 @@ impl Client {
                     .get(&redirect_location)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -671,7 +597,7 @@ impl Client {
             .bearer_auth(&self.token)
             .header(header::RANGE, &format!("bytes={}-{}", range.0, range.1))
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(request).await?;
 
@@ -682,13 +608,13 @@ impl Client {
                     .header(header::RANGE, &format!("bytes={}-{}", range.0, range.1))
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
         let status = response.status();
         if status.is_success() {
-            let resp_bytes = response.bytes().await.context(FetchBodyError)?;
+            let resp_bytes = response.bytes().await.context(FetchBodySnafu)?;
             Ok(resp_bytes.to_vec())
         } else {
             Err(extract_error(response).await)
@@ -706,7 +632,7 @@ impl Client {
                     .bearer_auth(&self.token)
                     .json(&query)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
         extract(response).await
@@ -721,7 +647,7 @@ impl Client {
             .delete(&url)
             .bearer_auth(&self.token)
             .build()
-            .context(RequestBuildError)?;
+            .context(RequestBuildSnafu)?;
 
         let redirect_location = self.request_with_redirect(request).await?;
 
@@ -731,7 +657,7 @@ impl Client {
                     .delete(&redirect_location)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -754,7 +680,7 @@ impl Client {
                     .get(&url)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -775,7 +701,7 @@ impl Client {
                         routing_config: routing_config.clone(),
                     })
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
@@ -795,7 +721,7 @@ impl Client {
                     .delete(&url)
                     .bearer_auth(&self.token)
                     .build()
-                    .context(RequestBuildError)
+                    .context(RequestBuildSnafu)
             })
             .await?;
 
