@@ -1,17 +1,11 @@
 mod builder;
 mod client;
-mod metadata_detector;
 mod parameters;
-mod profile;
 
 use builder::ClientBuilder;
 use parameters::Parameters;
 
-pub use client::Client;
+pub use client::{Client, ClientError};
 pub use interface::BlobMetaRequest as Meta;
 pub use interface::Type;
 pub use interface::{Expression, Query, QueryResponse};
-pub use profile::{Config, Profile};
-
-#[cfg(test)]
-mod test;
